#import <objects/Stream.h>
#import "Expr.h"
#import "alloc.h"

static inline const char *stringValue(id anObj)
{
	const char *typ;
	if(!anObj || ![anObj respondsToSelector:@selector(elementType)] || ![anObj respondsToSelector:@selector(element)]) return 0;
	typ = [anObj elementType];
	if(!typ || !(typ[0]=='*' || !typ[0]=='%')) return 0;
	return [anObj element].char_ptr_u;
}

@implementation Expr
- (elt)evalWith:state
{
	// Perform specified binary operations in order
	// Arg values are autoreleased; no special memory management needed.
	int i;
	int x, y;
	id val = [[self objectAtIndex:0] evalWith:state].id_u;
	x = [val element].int_u;
	if(ops[0] == '-') x = (-x);
	for(i=1; i<[self count]; i++)
	{
		y = [[[self objectAtIndex:i] evalWith:state].id_u element].int_u;
		switch(ops[i])
		{
			case '+': x += y; break;
			case '-': x -= y; break;
			case '*': x *= y; break;
			case '/': x /= y; break;
			default:  x  = y; break;
		}
	}
	// Reuse first arg as return (already autoreleased).
	[val setElement:ELT(int, x)];
	return ELT(id, val);
}
- (BOOL)isFinalized
{
	if(!ops) return NO;
	return [super isFinalized];
}
- finalizeWith:state
{
	int i, j, n = 0;
	const char *s = 0;
	id o, args = nil, replacement;

	// Do superclass finalization, if needed.
	if([self isFinalized]) return self;
	else if(!(replacement = [super finalizeWith:state])) return nil;
	else if(replacement != self) return replacement;
	
	// First syntax arg is list of operands and operations.
	// Merge list of ops and operands into ourselves, recording ops
	// and then clearing them out of parse tree.
	// Special case: if only one (+) arg, replace ourselves with it.
	if([self count]==0 || !(args = [self removeObjectAtIndex:0])
		|| (n = [args count])<=0 || !ALLOC(ops, char, (n+1)/2)) return nil;
	if(!prefix) prefix = '+';
	ops[0] = prefix;
	for(i=1, j=1; j<n; i++, j+=2)
	{
		// Skipping by twos to find ops.
		if((o = [args objectAtIndex:j]) && (s = stringValue(o)) && s[0]) ops[i] = s[0];
		else ops[i] = '+';
	}
	ops[i] = 0;
	for(j-=2; j>=0; j-=2) [args removeObjectAtIndex:j];
	[self empty];
	[self appendContentsOf:args];
	if([self count]==1 && ops[0]=='+') return [self removeObjectAtIndex:0];
	else return self;
}
- (void)dealloc { FREE(ops); [super dealloc]; }
- printIvarsTo:stream
{
	char *o;
	if(!ops || !*ops) return self;
	[super printIvarsTo:stream];
	if(ops[0] == '-') [stream writeFormat:" neg"];
	for(o=ops+1; *o; o++) [stream writeFormat:" %c", *o];
	return self;
}
@end


@implementation SgnExpr
- (BOOL)isFinalized
{
	if(!prefix) return NO;
	return [super isFinalized];
}
- finalizeWith:state
{
	// Preprocess extra initial optional syntax arg
	// for sign of first term.
	id opt;
	if([self isFinalized]) return self;
	if([self count]>0 && (opt = [self removeObjectAtIndex:0]) && [opt count]>0)
	{
		id o;
		const char *s = 0;
		if((o = [opt objectAtIndex:0]) && (s = stringValue(o)) && s[0]) prefix = s[0];
	}
	return [super finalizeWith:state];
}
@end


@implementation FirstExpr
- (BOOL)isFinalized
{
	// Exists only to replace it self with first expression arg.
	return NO;
}
- finalizeWith:state
{
	id o;
	int i, n = [self count];
	for(i=0; i<n; i++) if((o = [self objectAtIndex:i]) && ([o conformsToProtocol:@protocol(Parsing)] || ([o respondsToSelector:@selector(elementType)] && [o elementType][0]=='i'))) break;
	if(i >= n) return nil;
	return [[self removeObjectAtIndex:i] finalizeWith:state];
}
- (elt)evalWith:state
{
	// Exists only to replace it self with first expression arg.
	return ELT(id, nil);
}
@end


