// PValue.m
//
//----------------------------------------------------------------
// OOF Parser Package
// Copyright 1995 by Paul Burchard
// Distributed according to the GNU Library General Public License
//----------------------------------------------------------------

#import <Foundation/NSValue.h>
#import <objects/String.h>
#import <string.h>
#import "ParserAutoreleasePool.h"
#import "PValue.h"
#import "elt_p.h"
#import "alloc.h"

BOOL getStringValue(id anObj, const char **value)
{
	const char *typ;
	
	if(!anObj || !value) return NO;
	if([anObj respondsToSelector:@selector(cStringValue)])
		{ *value = [anObj cStringValue]; return YES; }
	else if([anObj respondsToSelector:@selector(cString)])
		{ *value = [anObj cString]; return YES; }
	else if([anObj respondsToSelector:@selector(elementType)]
		&& (typ = [anObj elementType]) && ENCODING_IS_STRING(typ))
	{
		if([anObj respondsToSelector:@selector(element)])
			{ *value = [anObj element].char_ptr_u; return YES; }
	}
	else if([anObj respondsToSelector:@selector(objCType)]
		&& (typ = [anObj objCType]) && ENCODING_IS_STRING(typ))
	{
		if([anObj respondsToSelector:@selector(getValue:)])
			{ [anObj getValue:(void *)value]; return YES; }
	}
	return NO;
}

BOOL getIntValue(id anObj, int *value)
{
	const char *typ;
	
	if(!anObj || !value) return NO;
	if([anObj respondsToSelector:@selector(intValue)])
		{ *value = [anObj intValue]; return YES; }
	else if([anObj respondsToSelector:@selector(elementType)]
		&& (typ = [anObj elementType])
		&& (*typ=='i' || *typ=='I'))
	{
		if([anObj respondsToSelector:@selector(element)])
		{
			*value = (*typ=='i' ? [anObj element].int_u
				: [anObj element].unsigned_int_u);
			return YES;
		}
	}
	else if([anObj respondsToSelector:@selector(objCType)]
		&& (typ = [anObj objCType])
		&& (*typ=='i' || *typ=='I'))
	{
		if([anObj respondsToSelector:@selector(getValue:)])
			{ [anObj getValue:(void *)value]; return YES; }
	}
	return NO;
}

@implementation PValue
- initElement:(elt)val type:(const char *)enc
{
	if(![super init]) return nil;
	if(!(type = enc)) { [self release]; return nil; }
	[self setElement:val];
	return self;
}
- init { return [self initElement:ELT(int, 0) type:@encode(int)]; }
- (void)dealloc
{
	if(ENCODING_IS_OBJECT(type)) [value.id_u autorelease];
	else if(ENCODING_IS_MUTABLE_STRING(type)) FREE(value.char_ptr_u);
	[super dealloc];
}
- autorelease
{
	[ParserAutoreleasePool addObject:self];
	return self;
}
//!!! Should also override +setAutoreleaseClass: and
//!!! +autoreleaseClass in newer versions of libobjects.
- copy
{
	// No [super deepenCopy] in OpenStep hierarchy.
	return [[super copy] deepenCopy];
}
- deepenCopy
{
	// No [super deepenCopy] in OpenStep hierarchy.
	if(ENCODING_IS_OBJECT(type)) value = ELT(id, [value.id_u copy]);
	else if(ENCODING_IS_MUTABLE_STRING(type) && value.char_ptr_u)
		value = ELT_STR(strdup(value.char_ptr_u));
	return self;
}
- (void)setElement:(elt)val
{
	if(ENCODING_IS_OBJECT(type))
	{
		[value.id_u autorelease];
		value = ELT(id, [val.id_u retain]);
	}
	else if(ENCODING_IS_MUTABLE_STRING(type))
	{
		FREE(value.char_ptr_u);
		if(val.char_ptr_u) value = ELT_STR(strdup(val.char_ptr_u));
	}
	else value = val;
}
- (elt)element { return value; }
- (const char *)elementType { return type; }
- (const char *)evalType { return @encode(id); }
- (BOOL)isFinalized { return YES; }
- finalizeWith:state { return self; }
- (elt)evalWith:state
{
	if(ENCODING_IS_OBJECT(type)) return ELT(id, [[value.id_u retain] autorelease]);
	else return ELT(id, [[self retain] autorelease]);
}
@end


