// ParserParsers.h
//
// These function-valued functions are the heart of the BNF parser.
//
// [...put syntax-semantics specs for these classes here...]
//
// Notice the difference between e.g. ParserLiteral and LiteralParser.
// ParserLiteral is a parser constructor which, given a string object
// read from the BNF, serves as a parser to detect that literal string.
// LiteralParser is a builtin parser that actually reads in the
// specification of a string literal from the input, and turns it into
// an object suitable for feeding to (for example) ParserLiteral.
//
//----------------------------------------------------------------
// OOF Parser Package
// Copyright 1995 by Paul Burchard
// Distributed according to the GNU Library General Public License
//----------------------------------------------------------------

#import <objects/Array.h>
#import <objects/String.h>
#import <objects/Stream.h>
#import "Parsing.h"

@interface ArgList:Array <Parsing>
// note designated initializer is -initWithType:capacity:
- removeNonParsing;
- defaultConstructorWith:state;
- printLispTo:stream;
- printIvarsTo:stream; // for subclassing -printLispTo:; use space-prefixed values
- traceObject:anObj with:state;
- deepenCopy; // for subclassing -copy
@end

@interface ParserSequence:ArgList
{
	BOOL isFinalSeq;
	BOOL isImmediate;
}
- (void)setFinalized:(BOOL)flag; // fake finalization to extent possible
- (const char *)cStringValue; // name of constructor (NULL => Default:)
- (BOOL)isImmediate;
- (void)setImmediate:(BOOL)flag;
@end

@interface ParserDefault:ParserSequence
{
	BOOL isFinalMerge;
}
@end

@interface ParserConstructor:ParserDefault
{
	char *constructorName;
}
- initWithCString:(const char *)ctorName;
- (void)setCStringValue:(const char *)ctorName;
- (const char *)cStringValue;
@end

@interface ParserSuffix:ArgList
@end

@interface ParserAlternation:ArgList
@end

@interface ParserRepetition:ArgList
{
	int lowerBound, upperBound;
}
- initWithLowerBound:(int)lb upperBound:(int)ub;
- (void)setLowerBound:(int)lb upperBound:(int)ub;
- (void)getLowerBound:(int *)lbp upperBound:(int *)ubp;
@end

@interface ParserOption:ArgList
@end

@interface ParserSymbol:ArgList
{
	char *symbolName;
}
- initWithCString:(const char *)symbName;
- (void)setCStringValue:(const char *)symbName;
- (const char *)cStringValue;
@end

@interface ParserLiteral:ArgList
{
	char *literal;
}
- initWithCString:(const char *)literalString;
- (void)setCStringValue:(const char *)literalString;
- (const char *)cStringValue;
@end

@interface ParserArgument:ArgList
{
	int argIndex;
}
- initWithInt:(int)argIdx;
- (void)setIntValue:(int)argIdx;
- (int)intValue;
@end

@interface ParserComposition:ArgList
{
	id function;
}
- function;
- setFunction:newFunction;
@end

@interface ParserDefinition:ArgList
@end

@interface ParserList:ArgList
@end

@interface SuffixParser:ArgList
- (const char *)cStringValue;
@end

@interface SuffixComposition:SuffixParser
@end

@interface SuffixRepetition:SuffixParser
@end

@interface SuffixSequence:SuffixParser
@end


