// etest.m

#import <stdio.h>
#import <stdlib.h>
#import <objects/StdioStream.h>
#import "Expr.h"

id newInputBufferForFilename(const char *fname)
{
	char *s = 0;
	FILE *istream;
	int n, len = 0;
	id iostate;

	if(!fname || !(istream = fopen(fname, "r"))) { fprintf(stderr, "Can\'t open %s\n", fname); exit(1); }
	if(!(s = malloc(BUFSIZ))) { fprintf(stderr, "Memory allocation failed\n"); exit(1); }
	while(1)
	{
		if(!(s = realloc(s, len+BUFSIZ))) { fprintf(stderr, "Memory allocation failed\n"); exit(1); }
		if((n = fread(s+len, 1, BUFSIZ-1, istream)) <= 0) break;
		len +=n;
	}
	s[len] = 0;
	fclose(istream);
	iostate = [[StatefulString alloc] initWithCString:s];
	if(!iostate) { fprintf(stderr, "Failed to create StatefulString\n"); exit(1); }
	return iostate;	
}

void main(int argc, char *argv[])
{
	char *s = 0;
	const char *t;
	int val;
	id ostream, state, iostate, diagnostics, bnf_boot, bnf, expr, p;

	// Create parsing state and output streams.
	state = [[PState alloc] init];
	ostream = [StdioStream standardOut];
	if(!ostream) { fprintf(stderr, "Can\'t open stdout\n"); exit(1); }
	diagnostics = [ostream retain];
	if(!diagnostics) { fprintf(stderr, "Can\'t open stdout\n"); exit(1); }
	
	// Read in the BNF syntax definition to override bootstrap versions.
	iostate = newInputBufferForFilename("bnf.bnf");
	[state putObject:[iostate autorelease] atName:"<"];
	////[state putObject:[diagnostics autorelease] atName:"2>"];
	bnf_boot = [state objectAtName:"bnf"];
	[bnf_boot evalWith:state];

	// Print out the new BNF parsers.
	[ostream writeFormat:"bnf := "];
	bnf = [state objectAtName:"bnf"]; [bnf printLispTo:ostream];
	[ostream writeFormat:"\n---\nbnf_statement := "];
	p = [state objectAtName:"bnf_statement"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n---\nbnf_parser := "];
	p = [state objectAtName:"bnf_parser"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n---\nbnf_term := "];
	p = [state objectAtName:"bnf_term"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n---\nbnf_factor := "];
	p = [state objectAtName:"bnf_factor"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n---\nbnf_atom := "];
	p = [state objectAtName:"bnf_atom"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n---\nlist := "];
	p = [state objectAtName:"list"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n-------------------------------\n"];

	// Read in the EXPR syntax definition using the new BNF parsers.
	iostate = newInputBufferForFilename("expr.bnf");
	[state push];
	[state putObject:[iostate autorelease] atName:"<"];
	////[state putObject:[diagnostics autorelease] atName:"2>"];
	[bnf evalWith:state];

	// Print out the new EXPR parsers.
	[ostream writeFormat:"\n-------------------------------\nexpr := "];
	expr = [state objectAtName:"expr"]; [expr printLispTo:ostream];
	[ostream writeFormat:"\n---\nsum := "];
	p = [state objectAtName:"sum"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n---\nterm := "];
	p = [state objectAtName:"term"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n---\nfactor := "];
	p = [state objectAtName:"factor"]; [p printLispTo:ostream];
	[ostream writeFormat:"\n-------------------------------\n"];

	// Read in EXPR's from stdin, then parse and evaluate them.
	////[state putObject:[diagnostics autorelease] atName:"2>"];
	if(!(s = malloc(BUFSIZ))) { fprintf(stderr, "Memory allocation failed\n"); exit(1); }
	while(gets(s))
	{
		iostate = [[StatefulString alloc] initWithCString:s];
		if(!iostate) { fprintf(stderr, "Failed to create StatefulString for \"%s\"\n", s); exit(1); }
		[state putObject:iostate atName:"<"];
		[ostream writeFormat:"---\nParsing: \"%s\"\n---\nFunction:  ", s];
		p = [[expr evalWith:state].id_u retain];
		if([p respondsTo:@selector(printLispTo:)])
			[p printLispTo:ostream];
		[ostream writeFormat:"\n---\nValue:  "];
		val = [[p evalWith:state].id_u element].int_u;
		[p release];
		[ostream writeFormat:"%d", val];
		[ostream writeFormat:"\n---\nUnparsed/Unseen:  \"%s\"/\"%s",
			((t=[[iostate currentContentsAsString] cStringValue])?t:""),
			((t=[[iostate futureContentsAsString] cStringValue])?t:"")];
		[ostream writeFormat:"\"\n-------------------------------\n"];
		[iostate release];
	}
	[state pop];
	[ostream release];
	[diagnostics release];
	exit(0);
}
